#
# Determine Drugs Similar to Disease using K-Means Clustering
#
# At this stage, we have finished preprocessing, and have produced a filtered
# LINCS matrix. This matrix still contains 1 row (the first) representing the
# disease z-scores for each important gene (columns). The remaining rows each
# represent an important drugs, with its z-score measurements.
#
#  Label    Gene1   Gene2   Gene3  ...  GeneN
#  Disease    x       x       x           x
#  Drug1      x       x       x           x
#  Drug2      x       x       x           x
#  ...
#  DrugN      x       x       x           x
#
# The goal of Drug Repositioning is to identify FDA approved drugs that may also
# be used to treat a different disease. To identify candidate drugs for a given 
# disease, we will determine the LINCS drugs that are "closest" to each breast
# cancer subtype (using gene z-score signatures). We employ K-Means to gauge
# closeness.
#
# Joel Rorseth
# March 30, 2019
#

library("rjson")
DATASET_DIR <- "/Users/Joel/Documents/Research/Drug Repositioning/Datasets"
setwd(DATASET_DIR)


write_mtrx_to_file <- function(df, dfname) {
  print(sprintf("Writing %s with dimension %d x %d", dfname, nrow(df), ncol(df)))
  filename <- paste0(dfname, ".csv")
  write.csv(df, file = filename, row.names = TRUE)
}

# Load the LINCS drug approval status lookup from JSON
load_lincs_approval_status <- function() {
  path <- paste(DATASET_DIR, "/lincs_approval_status.json", sep="")
  approval_file <- fromJSON(file = path)
  return(approval_file)
}

# Get the approval status of a drug, return N/A if not in lookup
get_status <- function(drug, approval_status) {
  if (drug %in% names(approval_status)) {
    return(approval_status[drug])
  } else {
    return("N/A")
  }
}

# Find the nearest cluster neighboring the cluster containing disease row
find_nearest_neighbor_cluster <- function(clusters, disease_cluster) {
  nearest_cluster <- 0
  nearest_cluster_dist <- 0
  
  # Determine closest cluster to disease cluster (go thru all k clusters)
  k <- nrow(clusters$centers)
  for (cl in c(1:k)) {
    
    if (cl == disease_cluster) { next }
    disease_clust_centroid <- clusters$centers[disease_cluster,]
    
    # Get the centroid for this cluster, find square dist from disease cluster
    neighbor_clust_centroid <- clusters$centers[cl,]
    clust_dist <- sum((disease_clust_centroid - neighbor_clust_centroid)^2)
    
    if (nearest_cluster_dist == 0 || clust_dist < nearest_cluster_dist) {
      nearest_cluster <- cl
      nearest_cluster_dist <- clust_dist
    }
  }
  
  print(sprintf(
    "Disease cluster %d only contains disease, finding nearest neighbor cluster", 
    disease_cluster))
  return(nearest_cluster)
}

# Calculate distance between a drug row and the disease row
get_row_dist <- function(cur_row, disease_row) {
  sum((cur_row[-1] - disease_row[-1])^2)
}


# Perform clustering using K-Means
cluster_disease_kmeans <- function(zscores_mtrx) {
  for (k in c(2:8)) {
    print(sprintf("Subtype %d, k=%d:", subtype, k))
    
    # Apply K-Means to zscore matrix, label rows with cluster
    clusters <- kmeans(zscores_mtrx, k, iter.max=100)
    cl_mtrx <- cbind(cl = clusters$cluster, zscores_mtrx)
    
    disease_cluster <- cl_mtrx["DISEASE","cl"]
    
    # If cluster containing disease only contains disease, merge w/ closest cluster
    if (clusters$size[disease_cluster] == 1) {
      nearest_cluster <- find_nearest_neighbor_cluster(clusters, disease_cluster)
      
      # Merge disease row into the nearest neighbor cluster (just reassign cluster)
      cl_mtrx["DISEASE","cl"] <- nearest_cluster
      disease_cluster <- cl_mtrx["DISEASE","cl"]
      print(sprintf("Disease has been merged into cluster %d", nearest_cluster))
    }
    
    print(sprintf("Disease is in cluster %d with size %d",
                  k, disease_cluster, sum(cl_mtrx[,"cl"] == disease_cluster)))
    
    
    # Filter matrix to only rows in the disease cluster
    cl_mtrx <- cl_mtrx[cl_mtrx[,"cl"] == disease_cluster,]
    print(sprintf("Removed non disease cluster rows, reduced from %d x %d to %d x %d", 
                  nrow(zscores_mtrx), ncol(zscores_mtrx), nrow(cl_mtrx), ncol(cl_mtrx)))
    
    # Sort the matrix by cluster, then rows in each cluster by within cluster dist    
    cl_mtrx <- cbind(dist = apply(cl_mtrx, 1, get_row_dist, cl_mtrx["DISEASE",]), cl_mtrx)
    cl_mtrx <- cl_mtrx[order(cl_mtrx[,1]),]
    
    
    # Write ordered cluster to CSV, with approval status
    approval_status <- load_lincs_approval_status()
    ordering <- cl_mtrx[,c(1,2)]
    
    
    # Append a column for approval status
    approval_col <- rep("NA", nrow(ordering))
    for (i in c(1:nrow(ordering))) {
      approval_col[i] <- get_status(rownames(ordering)[i], approval_status)
    }
    ordering <- cbind(ordering, approval_col)
    
    # Write to CSV
    filename <- paste0(as.character(k), "-means_subtype_", as.character(subtype))
    write_mtrx_to_file(ordering, filename)
    
  }
}


# Perform hierarchical clustering
cluster_disease_hier <- function(zscores_mtrx) {
  
  dist_mtrx <- dist(zscores_mtrx, method = 'euclidean')
  hclust_avg <- hclust(dist_mtrx, method = 'average')
  
  # Cut the dendogram into k trees, display cluster sizes
  # Note: This method appears to be unsuccessful since the disease sample always
  # seems to end up in its own cluster alone, for any given k
  row_clusters <- cutree(hclust_avg, k = 2)
  print(table(row_clusters))
}


for (subtype in c(1:10)) {
  filename <- paste(DATASET_DIR, "/Reduced-Zscores/reduced_zscores_subtype_", subtype, ".Rds", sep="")
  zscores_mtrx <- readRDS(filename)
  
  cluster_disease_kmeans(zscores_mtrx)
  cluster_disease_hier(zscores_mtrx)
}