#
# Create Anti-Correlation Matrix
#
# The goal of this script is to essentially further reduce the disease / drug
# z-score matrix (all_zscores_subtype_i.Rds), by removing drugs and genes that
# are of lesser significance to this particular breast cancer subtype. The 
# reduced matrices are written to (reduced_zscores_subtype_i.Rds).
#
# Generate an anti-correlation matrix by determining up/down regulation amongst
# the cancer sample z-scores and the z-scores of the LINCS drugs. Then, remove 
# drugs and genes from the z-score matrix, whose corresponding row / column in 
# the anti-correlation matrix have the most 0's.
#
# Joel Rorseth
# Jan 5, 2019
#

DATASET_DIR <- "/Users/Joel/Documents/Research/Drug Repositioning/Datasets"
setwd(DATASET_DIR)


write_mtrx_to_file <- function(df, dfname) {
  print(sprintf("Writing %s with dimension %d x %d", dfname, nrow(df), ncol(df)))
  filename <- paste0(dfname, ".Rds")
  saveRDS(df, file=filename)
}


# Obtain a vector with count of zeroes for each row in a df
count_zeroes_rows_mtrx <- function(df) {
  return(apply(df, 1, count_zeroes_vec))
}

count_zeroes_cols_mtrx <- function(df) {
  return(apply(df, 2, count_zeroes_vec))
}

# Count the number of zeroes in a vector (row or col)
count_zeroes_vec <- function(vec) {
  return(sum(vec == 0))
}


# Remove the worst 'p'% rows, most populated with 0's
filter_zero_rows_percent <- function(df, p) {

  # Sort df rows by # of 0's, with fewest 0's in rows at the top
  df <- df[order(count_zeroes_rows_mtrx(df)),]
  
  # Display stats about # 0's
  least_zeroes <- count_zeroes_rows_mtrx(df)[1]
  most_zeroes <- tail(count_zeroes_rows_mtrx(df), 1)
  print(sprintf("Number of zeroes in rows range from %d to %d", least_zeroes, most_zeroes))
  
  # Keep first (1-p)% rows (remove worst p% rows w/ most 0's)
  print(sprintf("Removing %f percent of the rows with the most 0's", p))
  best_n <- nrow(df) - as.integer(nrow(df) * p)
  df <- df[c(1:best_n),]
  
  # Display stats about # 0's
  least_zeroes <- count_zeroes_rows_mtrx(df)[1]
  most_zeroes <- tail(count_zeroes_rows_mtrx(df), 1)
  print(sprintf("Taking best %d rows, now range is %d to %d", best_n, least_zeroes, most_zeroes))
  
  print(sprintf("-> Finished filtering rows"))
  return(df)
}


# Remove k worst cols (most 0's), where k = # of 0's in the best row
filter_zero_cols_relative <- function(df) {
  
  # Get # of 0's in best row
  df <- df[order(count_zeroes_rows_mtrx(df)),]
  k <- count_zeroes_rows_mtrx(df)[1]
  print(sprintf("The best row has %d zeroes", k))
  
  # Sort columns by # 0's (best columns with fewest 0's are leftmost)
  df <- df[, order(count_zeroes_cols_mtrx(df))]
  
  # Display stats about # 0's
  least_zeroes <- count_zeroes_cols_mtrx(df)[1]
  most_zeroes <- tail(count_zeroes_cols_mtrx(df), 1)
  print(sprintf("Number of zeroes in cols range from %d to %d", least_zeroes, most_zeroes))
  
  # Keep best (ncol-k) columns (remove rightmost k cols)
  print(sprintf("Removing %d of the cols with the most 0's", k))
  best_n <- ncol(df) - k
  df <- df[, c(1:best_n)]
  
  # Display stats about # 0's
  least_zeroes <- count_zeroes_cols_mtrx(df)[1]
  most_zeroes <- tail(count_zeroes_cols_mtrx(df), 1)
  print(sprintf("Taking best %d cols, now range is %d to %d", best_n, least_zeroes, most_zeroes))
  
  print(sprintf("-> Finished filtering columns"))
  return(df)
}


# Create the anti-correlation matrix by cross-examining drug z-scores w/ disease
create_anti_corr_mtrx <- function(zscores_mtrx) {
  
  # Create new df of zeroes
  ac_mtrx <- as.data.frame(matrix(0, nrow=(nrow(zscores_mtrx)-1), ncol=ncol(zscores_mtrx)))
  
  # Compare each zscore of each drug profile to the disease profile (first row)
  for (r in c(2:nrow(zscores_mtrx))) {
    for (c in c(1:ncol(zscores_mtrx))) {
      
      disease_z <- zscores_mtrx[1,c]
      drug_z <- zscores_mtrx[r,c]
      
      # Opposite signs for zscore value should yield +/- 1
      if (disease_z < 0 && drug_z > 0) {
        ac_mtrx[r-1,c] <- 1
      }
      
      if (disease_z > 0 && drug_z < 0) {
        ac_mtrx[r-1,c] <- -1
      }
    }
  }
  
  # Set rownames and colnames (drug names)
  rownames(ac_mtrx) <- rownames(zscores_mtrx)[c(2:nrow(zscores_mtrx))]
  colnames(ac_mtrx) <- colnames(zscores_mtrx)
  
  return(ac_mtrx)
}


for (subtype in c(1:10)) {
  filename <- paste(DATASET_DIR, "/All-Subtype-Zscores/all_zscores_subtype_", subtype, ".Rds", sep="")
  zscores_mtrx <- readRDS(filename)
  
  print(sprintf("For subtype %d, zscore mtrx is %d x %d", subtype, 
    nrow(zscores_mtrx), ncol(zscores_mtrx)))
  
  # Get anti-correlation matrix, remove rows with the most zeroes (remove 50%)
  anti_corr_mtrx <- create_anti_corr_mtrx(zscores_mtrx)
  anti_corr_mtrx <- filter_zero_rows_percent(anti_corr_mtrx, 0.50)
  anti_corr_mtrx <- filter_zero_cols_relative(anti_corr_mtrx)
  
  reduced_zscores_matrix <- zscores_mtrx[
    rownames(zscores_mtrx) %in% union(rownames(anti_corr_mtrx), "DISEASE"),
    colnames(zscores_mtrx) %in% colnames(anti_corr_mtrx)
  ]
  
  # Write the anti correlation matrix to file (not important, could omit)
  write_mtrx_to_file(anti_corr_mtrx, 
                     paste0("anti_corr_mtrx_subtype_", as.character(subtype)))
  # Write new zscore matrix reduced using anti correlation matrix
  write_mtrx_to_file(reduced_zscores_matrix, 
                     paste0("reduced_zscores_subtype_", as.character(subtype)))
  print("")
}

