#
# Z-score matrix creation
#
# At this point, we now have, for each of the 10 breast cancer subtypes, a dataset
# associating certain genes (particular to that subtype) with a z-score. This is
# the disease data z-scores. This script will create 10 matrices, where for
# each subtype, we combine these disease z-scores (the first row of new matrix) 
# with the LINCS z-scores for the same genes in certain drugs (the remaining 
# rows, one for each drug). The matrices are written to (all_zscores_subtype_i.Rds).
#
#  Label    Gene1   Gene2   Gene3  ...  GeneN
#  Disease    x       x       x           x
#  Drug1      x       x       x           x
#  Drug2      x       x       x           x
#  ...
#  DrugN      x       x       x           x
#
#
# Joel Rorseth
# December 4, 2018
#

DATASET_DIR <- "/Users/Joel/Documents/Research/Drug Repositioning/Datasets"
LINCS_PATH <- "Lincs-Breast-Cancer.RData"
DIS_ZSCORES_DIR <- "Disease-Subtype-Zscores/"

setwd(DATASET_DIR)
load(LINCS_PATH)

print(sprintf("Loaded Drugs with size %d x %d", nrow(Drugs), ncol(Drugs)))
print(sprintf("Loaded Genes with size %d x %d", nrow(Genes), ncol(Genes)))
print(sprintf("Loaded Lincs with size %d x %d", nrow(Lincs), ncol(Lincs)))

# Filter Drugs by taking sample w/ most common dose, cell id and pert_itime
# Remove the few (~100) dups to achieve 1 drug for each pert_iname
Drugs <- subset(Drugs, pert_idose == "10.0 um" & cell_id == "MCF7" & pert_itime == "24 h")
Drugs <- Drugs[!duplicated(Drugs$pert_iname),]
Lincs <- Lincs[, as.numeric(rownames(Drugs))]

print("After filtering to only unique drugs...")
print(sprintf("Drugs has size %d x %d", nrow(Drugs), ncol(Drugs)))
print(sprintf("Genes has size %d x %d", nrow(Genes), ncol(Genes)))
print(sprintf("Lincs has size %d x %d", nrow(Lincs), ncol(Lincs)))

# LINCS rows are genes (to be filtered by subtype), and cols are drugs (now filtered)
rownames(Lincs) <- Genes$V1
colnames(Lincs) <- Drugs$pert_iname
write.csv(colnames(Lincs), file = "all_lincs_drug_names.csv", row.names = FALSE)



write_df_to_file <- function(df, subtype) {
  print(sprintf("Combined and transposed, final matrix is %d x %d", nrow(df), ncol(df)))
  filename <- paste0("all_zscores_subtype_", as.character(subtype), ".Rds")
  saveRDS(df, file=filename)
}


combine_disease_drugs <- function(disease, drugs, subtype) {
  
  # Sort the rows of both matrices to align matching genes
  disease <- disease[order(disease[,1]), ]
  drugs <- drugs[order(row.names(drugs)), ]
  
  # Combine zscore col (2) from disease and zscores of each drug into new df
  combined <- rbind(disease[,2], t(drugs))
  
  # Colnames and rownames are set automatically (inherited from Lincs), label disease row
  rownames(combined)[1] <- "DISEASE"
  
  write_df_to_file(combined, subtype)
}


create_matrix_for_subtype <- function(subtype) {
  print(sprintf("Creating matrix for subtype %d:", subtype))
  
  # Load in disease z-scores for subtype genes
  filename <- paste(DIS_ZSCORES_DIR, "subtype_", subtype, "_zscores.csv", sep="")
  disease_zscores <- read.csv(filename, header=T)
  
  # Reduce LINCS rows to only include genes important to this subtype
  all_subtype_genes <- as.vector(disease_zscores[,1])
  subtype_Lincs <- Lincs[rownames(Lincs) %in% all_subtype_genes,]
  
  print(sprintf("Of the %d subtype genes, Lincs measures %d of them", 
    length(all_subtype_genes), nrow(subtype_Lincs)))
  
  # Reduce the disease z-scores to only genes which were found in LINCS
  disease_zscores <- disease_zscores[disease_zscores[,1] %in% rownames(Lincs),c(1,3)]
  
  combine_disease_drugs(disease_zscores, subtype_Lincs, subtype)
  print("Done")
}


# Main entry point
for (subtype in c(1:10)) {
  create_matrix_for_subtype(subtype)
}
