#
# Gene Filtering script
#
# For each of the 10 breast cancer subtypes, filter the gene expression data
# to include only genes listed as network biomarkers for that subtype. This
# should yield 10 matrices each corresponding to a subtype. Calculate a z-score
# for each gene in each subtype, by determining diploid gene measurements and
# averaging the expression values.
#
# Thus for each subtype, we obtain a filtered set of genes (significant to this 
# breast cancer subtype), with each a calculated z-score. These assignments 
# are written to (subtype_i_zscores.csv).
#
# Joel Rorseth
# November 5, 2018
#

require(gdata)

DATASET_DIR <- "/Users/Joel/Documents/Research/Drug Repositioning/Datasets"
EXPR_MTRX_PATH <- "brca_metabric/data_RNA_Seq_expression_median.txt"
CNA_MTRX_PATH <- "brca_metabric/data_CNA.txt"
BIOM_DIR <- "Ten-Network Biomarkers/"
setwd(DATASET_DIR)


# MARK: TESTING
# Test sample data, see https://github.com/cBioPortal/cbioportal/blob/master/docs/Z-Score-normalization-script.md
run_tests <- function() {
  r1 <- c("Gene1", 6205, 0.765, 0.716, 0.417125, 0.115, 0.492875, -0.525, -0.169, 0.396, 0.50475, 0.400875, 0.393125, 0.9165, 0.627125, 0.337125, 0.705, 0.16425, 0.325, 0.11175)
  r2 <- c("Gene1",6205,0,0,0,1,1,0,-1,0,0,2,0,0,1,-1,0,0,-1,0)
  temp_exp_m <- rbind.data.frame(r1)
  temp_cna_m <- rbind.data.frame(r2)
  colnames(temp_exp_m) <- c("HUGO","ENTREZ","P1","P2","P3","P4","P5","P6","P7","P8","P9","P10","P11","P12","P13","P14","P15","P16","P17","P18")
  colnames(temp_cna_m) <- c("HUGO","ENTREZ","P1","P2","P3","P4","P5","P6","P7","P8","P9","P10","P11","P12","P13","P14","P15","P16","P17","P18")
  find_zscores(temp_exp_m, temp_cna_m, 99)  
}


write_df_to_file <- function(subtype_zscores_df, subtype) {
  # Output CSV for given subtype will list rows of genes, each with a z-score
  filename <- paste0("subtype_", as.character(subtype), "_zscores.csv")
  write.table(subtype_zscores_df, file=filename, row.names=F, na="NA", 
              col.names=c("Hugo_Symbol","Entrez_Gene_Id","Zscore"), sep=",", quote=F)
}


find_zscores <- function(expr, cna, subtype) {
  subtype_zscores_df <- data.frame(expr[,1], expr[,2], integer(length(expr[,1])))
  
  # For each gene applicable to this subtype...
  for (i in 1:nrow(expr)) {
    
    # Get expr values for patients who recorded diploid on the current gene
    diploid_patients <- colnames(cna)[which(cna[i,] == 0)]
    diploid_patients <- diploid_patients[diploid_patients %in% colnames(expr)]
    diploid_expr_values <- expr[i, diploid_patients]
    diploid_expr_values <- as.numeric(as.character(unlist(diploid_expr_values)))
    
    # Calculate mean and standard deviation of these diploid expr values
    dip_mean <- mean(diploid_expr_values)
    dip_stddev <- sd(diploid_expr_values)
    
    # Apply formula to every raw expr value (even non-diploid) to get z-scores
    #  z-score = (raw_expr_val - mean) / std_dev
    zscores <- lapply(as.numeric(as.character(unlist(expr[i,-c(1,2)]))), 
                      function(r) (r-dip_mean)/dip_stddev)
    
    # Store average of z-scores for this gene, as the z-score of the gene itself
    gene_avg_zscore <- mean(as.numeric(zscores))
    subtype_zscores_df[i, 3] = gene_avg_zscore
  }
  
  write_df_to_file(subtype_zscores_df, subtype)
  print(sprintf("Finished calculating z-scores for genes for subtype %d", subtype))
}


# Read gene expression matrix, remove any duplicate genes measured (only 1)
expressions <- read.table(EXPR_MTRX_PATH, header=TRUE, fill=TRUE)
expressions <- expressions[!duplicated(expressions[,1]),]
cna_data <- read.table(CNA_MTRX_PATH, header=TRUE, fill=TRUE)

# TODO: Consider removing any row (gene) from expr matrix with any NA values

for (subtype in c(1:10)) {
  biom_filename <- paste(BIOM_DIR, "Subtype-", subtype, ".xls", sep="")
  biomarkers <- read.xls(biom_filename, header=FALSE)
  
  # Get set of genes whom are connected with weight >= 50%
  biomarkers <- biomarkers[biomarkers[,3] >= 50,]
  biom_genes_set <- union(unique(biomarkers[, 1]), unique(biomarkers[, 2]))
  
  # Filter rows to those where gene has row in CNA matrix and is in biomarker set
  sub_expressions <- expressions[expressions[,1] %in% biom_genes_set,]
  
  print(sprintf("Subtype %d:", subtype))
  print(sprintf("Of %d genes in biomarker set, %d found in expr matrix", 
                length(biom_genes_set), length(sub_expressions[,1])))
  
  sub_expressions <- sub_expressions[sub_expressions[,1] %in% cna_data[,1],]
  
  print(sprintf("Of these selected genes, only %d have recorded CNA data", 
                length(sub_expressions[,1])))
  print(sprintf("Expression matrix dim is now %d x %d", 
                length(sub_expressions[,1]), length(sub_expressions[1,])))
  
  # Pass off filtered expression matrix to find z-scores
  find_zscores(sub_expressions, cna_data, subtype)
  print(" ")
}
